#####--------------KRIS PRUITT---------------------------------#####

#-------------------------------------------------------------------
#  		FUEL CELL (Combined Heat and Power) & GRID
#		Total Cost Objective
#-------------------------------------------------------------------

#-------------------------------------------------------------------
#  Sets
#-------------------------------------------------------------------

set N;						#set of all months
set Tn{N};					#set of all hours in month n
set T = union {n in N} Tn[n];			#set of all hours

#-------------------------------------------------------------------
#  Parameters
#-------------------------------------------------------------------

param cd{n in N};				#peak demand charge for electricity, month n
param ce{t in T};				#price of elec. from grid, hour t
param cg{t in T};				#energy charge for natural gas, hour t
param tau;					#time increment per period in hours
param de{t in T};				#electricity demand in period t
param dh{t in T};				#heat demand in period t
param gpx;					#max power capacity of power-only fcells
param hpx;					#max power capacity of CHP fcells
param vpx;					#max power capacity of PV cells
param pvf{t in T};				#forecasted percent max solar, period t
param gmt;					#max turn-down power-only fcells
param hmt;					#max turn-down CHP fcells
param gru;					#max ramp-up rate for power-only fcell
param grd;					#max ramp-down for power-only fcell
param hru;					#max ramp-up rate for CHP fcell
param hrd;					#max ramp-down for CHP fcell
param gaq;					#$/kW of power-only fuel cell
param haq;					#$/kW of CHP fuel cell
param vaq;					#$/kW of PV cell
param gom;					#variable o&m cost power fcell
param hom;					#variable o&m CHP fcell
param vom;					#variable o&m pv cell
param bom;					#variable o&m boiler
param waq;					#$/gal of tank
param wx;					#initial capacity of tank
param wxx;					#max tank capacity
param wd;					#decay rate of water temp
param we;					#heat x efficiency
param wu;					#max water temp
param wh;					#hot water delivery temp
param wr;					#average return water temp
param wl;					#water main temp
param ws;					#specific heat of water
param sx;					#max storage capacity of battery
param msc;					#min state-of-charge of battery
param scr;					#max charge rate for battery
param sdr;					#max discharge rate for battery
param sce;					#charge efficiency for battery
param sde;					#discharge efficiency for battery
param saq;					#$/kWh of battery
param fe;					#elec eff during start-up
param fh;					#CHP exhaust output per unit natural gas input
param ft;					#average temp of CHP exhaust
param fs;					#specific heat of exhaust
param fb;					#heat efficiency for boiler
param zt;					#carbon tax
param zu;					#carbon emission rate from grid
param zg;					#carbon emission rate of natural gas

#-------------------------------------------------------------------
#  Variables
#-------------------------------------------------------------------

var mE{n in N} >= 0, :=max{t in Tn[n]}de[t];	#max supplied by grid in month n
var mEP{t in T} >=0, :=de[t];			#power supplied by grid in period t
var mES{t in T} >=0, :=0;			#power sold back to grid in t
var mW{t in T} >=0, :=wr;			#temp of tank water
var mWX{t in T} binary, :=0;			#1 if temp is above return temp
var mWY{t in T} binary, :=0;			#1 if temp is above delivery temp
var mWA >=0, :=wx;				#volume of water tank
var mWB binary, :=0;				#1 if tank is acquired
var mWD{t in T} >=0, :=dh[t]/(ws*(wh-wr));	#flowrate of water out of tank
var mWC{t in T} >=0, :=0;			#flowrate of exhaust into tank
var mBNG{t in T} >=0, :=dh[t]/fb;		#nat gas into boiler
var mGA >=0, integer, :=0;			#num power-only fcells acquires
var mHA >=0, integer, :=0;			#num CHP fcells acquired
var mVA >=0, integer, :=0;			#num PV cells acquired
var mSA >=0, integer, :=0;			#num batteries acquired
var mGN{t in T} >=0, integer, :=0;		#num power-only fcells turned on period t
var mHN{t in T} >=0, integer, :=0;		#num CHP fcells turned on period t
var mGY{t in T:t>1} >=0, :=0;			#num power fcells turned on t to t-1
var mHY{t in T:t>1} >=0, :=0;			#num CHP fcells turned on t to t-1
var mP{t in T} >=0, :=0;			#power from power-only fuel cells in t
var mCP{t in T} >=0, :=0;			#power from CHP fuel cells in t
var mV{t in T} >=0, :=0;			#power from PV cells in t
var mSC{t in T} >=0, :=0;			#state-of-charge of batteries in t
var mCR{t in T} >=0, :=0;			#charge rate in t
var mDR{t in T} >=0, :=0;			#depletion rate in t
var mGE{t in T} >= fe, :=fe;			#elec eff of power-only fcells
var mHE{t in T} >= fe, :=fe;			#elec eff of CHP fcells
var mGNG{t in T} >=0, :=0;			#gas for power-only fcells
var mHNG{t in T} >=0, :=0;			#gas for CHP fcells


#---------------------------------------------------------
#		Objective Function
#---------------------------------------------------------

minimize minlp_cost:	
gaq*gpx*mGA + haq*hpx*mHA + vaq*vpx*mVA + saq*sx*mSA + waq*(mWA-wx) +
tau*sum{t in T}((bom*fb+cg[t]+zt*zg)*mBNG[t])+
tau*sum{t in T}(gom*mP[t] + hom*mCP[t] + vom*mV[t]) +
sum{t in T:t>1}((cg[t]+zt*zg)*(((gmt*gpx)/fe)*mGY[t]+((hmt*hpx)/fe)*mHY[t]))+ 
tau*sum{t in T}((cg[t]+zt*zg)*(mGNG[t]+mHNG[t])) +
tau*sum{t in T}((ce[t]+zt*zu)*mEP[t])- tau*sum{t in T}(ce[t]*mES[t]) +
(sum{n in N}(cd[n]*mE[n]));


#----------------------------------------------------------
#		Constraints
#----------------------------------------------------------

# ... determine the maximum macrogrid demand in each month
subject to mminmax{n in N,t in Tn[n]}: mE[n] >= mEP[t];

# ... cannot be a net generator each month
subject to mnetgen{n in N}: sum{t in Tn[n]}mES[t] <= sum{t in Tn[n]}mEP[t];

# ... power and heat demand met by DG and grid
subject to mdemande{t in T}: mP[t] + mCP[t] + mV[t] + (sde*mDR[t]-mCR[t]) + (mEP[t]-mES[t]) = de[t];
subject to mdemandh{t in T}: - mWD[t]*(ws*(wh-wr))*wl + (ws*(wh-wr))*mW[t]*mWD[t] = dh[t]*(mW[t]-wl-mW[t]*mWY[t]+wh*mWY[t]);

# ... max power from PV cells
subject to mmaxpowv{t in T}: mV[t] <= pvf[t]*vpx*mVA;

# ... max and min power from fuel cells
subject to mpow_lo{t in T}: mP[t] >= gmt*gpx*mGN[t];
subject to mpow_hi{t in T}: mP[t] <= gpx*mGN[t];
subject to mpow_buyon{t in T}: mGA >= mGN[t]; 
subject to mchp_lo{t in T}: mCP[t] >= hmt*hpx*mHN[t];
subject to mchp_hi{t in T}: mCP[t] <= hpx*mHN[t];
subject to mchp_buyon{t in T}: mHA >= mHN[t]; 

# ... ramp-up and ramp-down rates for fcells
subject to mpow_rup{t in T:t<card(T)}: mP[t+1] - mP[t] <= tau*gru*mGN[t+1];
subject to mpow_rdown{t in T:t<card(T)}: mP[t] - mP[t+1] <= tau*grd*mGN[t];
subject to mchp_rup{t in T:t<card(T)}: mCP[t+1] - mCP[t] <= tau*hru*mHN[t+1];
subject to mchp_rdown{t in T:t<card(T)}: mCP[t] - mCP[t+1] <= tau*hrd*mHN[t];

# ... number of fcells that start up between t and t-1
subject to mpow_oninc{t in T:t>1}: mGN[t]-mGN[t-1] <= mGY[t];
subject to mchp_oninc{t in T:t>1}: mHN[t]-mHN[t-1] <= mHY[t];

# ... natural gas consumption
subject to mggas{t in T}: mGE[t]*mGNG[t] = mP[t];
subject to mhgas{t in T}: mHE[t]*mHNG[t] = mCP[t];
subject to mbgas{t in T}: mBNG[t] = (ws/fb)*(wh*mWD[t]-wh*mWY[t]*mWD[t]-mW[t]*mWD[t]+mWD[t]*mW[t]*mWY[t]);

# ... electric efficiency of the fcells
subject to mpow_eff{t in T}: mGE[t]*mGN[t] = 0.61*mGN[t]-0.02*mP[t];
subject to mchp_eff{t in T}: mHE[t]*mHN[t] = 0.61*mHN[t]-0.02*mCP[t];

# ... temp of water tank
subject to mtanktemp{t in T:t>1}: ws*mW[t]*mWA - ws*mW[t-1]*mWA + ws*wd*mWA*mWX[t-1]*mW[t-1] = tau*we*fs*ft*mWC[t-1] - tau*we*fs*mW[t-1]*mWC[t-1] - tau*ws*mW[t-1]*mWD[t-1] + tau*ws*wr*mWD[t-1];
subject to mtemp1{t in T}: mW[t] <= wr + (wu-wr)*mWB;
subject to mtemp3{t in T}: mW[t] <= wr + 0.1 + (wu-wr-0.1)*mWX[t];
subject to mtemp4{t in T}: mW[t] >= wr + 0.1*mWX[t];
subject to mtemp5{t in T}: mW[t] <= wh+(wu-wh)*mWY[t];
subject to mtemp6{t in T}: mW[t] >= wh-(wh-wr)*(1-mWY[t]);
subject to mtemp7: mW[1] = mW[card(T)];

# ... heating water tank
subject to mtankheat1{t in T}: mWC[t] <= fh*mHNG[t];

# ... water tank acquisition
subject to mtankacq1: mWB <= mHA;
subject to mtankacq2: ceil((max{t in T}de[t])/hpx)*mWB >= mHA;
subject to mtanksize1: mWA >= wx;
subject to mtanksize2: mWA <= wxx;

# ... state-of-charge of the battery
subject to msoc{t in T:t>1}: mSC[t] = mSC[t-1] + tau*(sce*mCR[t-1]-mDR[t-1]);
subject to msoc_init: mSC[1] = mSC[card(T)];

# ... state-of-charge max and min capacity
subject to mst_upcap{t in T:t>1}: mSC[t] <= sx*mSA;
subject to mst_locap{t in T:t>1}: mSC[t] >= msc*sx*mSA;

# ... battery max charge and deplete rates
subject to mcharge{t in T}: mCR[t] <= scr*mSA;
subject to mdeplete{t in T}: mDR[t] <= sdr*mSA;













